/* 
Inspired by junesiphone's jstorage.js
Script by Paras Khanchandani https://twitter.com/ParasKCD

#Usage:-

- Initialization:
localstore.init({
    storageName: //some name for storage
});

- Adding a Value:
localstore.addValue('someValueName', 'value');

- Removing a Value:
localstore.removeValue('someValueName');

- XenHTML specific
    - Adding an app:
    localstore.addApp('arrayName', 'app');

    - Removing an app:
    localstore.removeApp('arrayName', 'app');

    - Replacing an app:
    localstore.replaceApp('arrayName', 'oldApp', 'newApp');
*/

var localstore = {
    storageName: "",
    storageData: [],
    load: function(data) {
        let storage = data;
        if(storage) {
            for(let i = 0; i < localstore.storageData.length; i++) {
                localstore[localstore.storageData[i]] = storage[localstore.storageData[i]];
            }
        }
    },
    save: function() {
        let storage = {};
        for(let i = 0; i < this.storageData.length; i++) {
            storage[localstore.storageData[i]] = localstore[localstore.storageData[i]];
        }
        api.fs.write('/var/mobile/Documents/' + localstore.storageName + '.json', storage).catch((error) => console.log(error));
    },
    addApp: function(arrayName, app) {
        if(localstore[arrayName]) {
            if(localstore[arrayName].indexOf(app) > -1) {
                alert('App already placed');
                return;
            } else {
                localstore[arrayName].push(app);
            }
        } else {
            if(Array.isArray(app)) {
                localstore[arrayName] = app;
            } else {
                localstore[arrayName] = [app];
            }
        }
        localstore.storageData.push(arrayName);
        this.save();
    },
    replaceApp: function(arrayName, older, newer) {
        if(localstore[arrayName].indexOf(newer) > -1) {
            alert('App already placed');
            return;
        } else {
            let index = this[arrayName].indexOf(older);
            if(index !== -1) {
                this[arrayName][index] = newer;
            }
            this.save();
        }
    },
    removeApp: function(arrayName, app) {
        let index = localstore[arrayName].indexOf(app);
        localstore[arrayName].splice(index, 1);
        this.save();
        if(localstore[arrayName].length == 0) {
            localstore.removeValue(arrayName);
        }
    },
    addAppObj: function(arrayName, app) {
        if(localstore[arrayName]) {
            if(localstore[arrayName].map((e) => e.identifier).indexOf(app.identifier) > -1) {
                alert("App already placed");
                return;
            } else {
                localstore[arrayName].push(app);
            }
        } else {
            if(Array.isArray(app)) {
                localstore[arrayName] = app;
            } else {
                localstore[arrayName] = [app];
            }
        }
        localstore.storageData.push(arrayName);
        this.save();
    },
    removeAppObj: function(arrayName, app) {
        let index = localstore[arrayName].map((e) => e.identifier).indexOf(app);
        localstore[arrayName].splice(index, 1);
        this.save();
        if(localstore[arrayName].length == 0) {
            localstore.removeValue(arrayName);
        }
    },
    modifyAppObj: function(arrayName, appID, app) {
        let index = localstore[arrayName].map((e) => e.identifier).indexOf(appID);
        localstore[arrayName][index] = app;
        this.save();
    },
    changeAppOrder: function(arrayName, appA, appB) {
        let indexA = this[arrayName].indexOf(appA);
        let indexB = this[arrayName].indexOf(appB);
        this[arrayName][indexA] = appB;
        this[arrayName][indexB] = appA;
        this.save();
    },
    changeAppPosition: function(arrayName, locationPrevious, locationNew) {
        let numberOfDeletedElm = 1;
        const elm = this[arrayName].splice(locationPrevious, numberOfDeletedElm)[0];
        numberOfDeletedElm = 0;
        this[arrayName].splice(locationNew, numberOfDeletedElm, elm);
        this.save();
    },
    addValue: function(name, value) {
        localstore[name] = value;
        localstore.storageData.push(name);
        this.save();
    },
    removeValue: function(name) {
        localstore[name] = null;
        let index = this.storageData.indexOf(name);
        this.storageData.splice(index, 1);
        this.save();
    },
    resetStorage: function() {
        localStorage.removeItem(localstore.storageName);
        location.href = location.href;
    },
    init: function(params) {
        this.storageName = params.storageName;
        api.fs.exists('/var/mobile/Documents/' + localstore.storageName + '.json').then((exists) => {
            if(exists) {
                api.fs.read('/var/mobile/Documents/' + localstore.storageName + '.json').then((data) => {
                    localstore.storageData = Object.keys(data);
                    localstore.load(data);
                    params.callback();
                }).catch((error) => console.log(error));
            } else {
                localstore[localstore.storageName] = [];
            }
        });
    }
}